@@ -12,30 +12,35 @@ import java.io.FileWriter;
 
 //Admittedly a little sloppy, but I don't want to spend forever on this...
 
-public class ControllerGUI extends JFrame implements ActionListener, KeyListener {
+public class ControllerGUI extends JFrame implements ActionListener, KeyListener, EStopListener.UpdatedListener {
+	org.tekkotsu.sketch.SketchGUI cameraSkGUI=null, localSkGUI=null, worldSkGUI=null;
 	JList menu;
 	JScrollPane menuScroll;
 	JComboBox title;
 	JLabel status;
 	JList scripts;
 	ControllerListener comm;
+	EStopListener estopComm;
 	JButton backBut;
 	JButton refreshBut;
 	JButton reconnectBut;
-	EStopButton estop;
+	EStopPanel estop;
+	JButton estopBut;
 	JTextField inputField;
 	boolean isUpdating=false;
 	DefaultListModel scriptsModel=new DefaultListModel();
-  Vector inputFieldHistory;
-  int inputFieldLocation;
+	Vector inputFieldHistory;
+	int inputFieldLocation;
 	static Preferences prefs = Preferences.userNodeForPackage(ControllerGUI.class);
 	final static int MAX_STORE_INPUT_HIST=20;
 	
 	ControllerGUI(String addr, int port) {
 		super();
+		setTitle("TekkotsuMon: Controller ("+addr+")");
 		comm = new ControllerListener(addr,port);
-		comm.listener=this;
 		init();
+		comm.listener=this;
+		comm.needConnection();
 	}
 
 	// Disables the component if there's nothing selected to apply it to
@@ -88,7 +93,9 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 			title.setEnabled(list.isEnabled());
 			title.setFont(list.getFont());
 			title.setOpaque(true);
-			title.setPreferredSize(new Dimension(list.getSize().width,title.getPreferredSize().height));
+			//I would rather use title.getPreferredSize().height instead of the hard-coded 15
+			//but sometimes values around 16390 are returned, and sometimes 0 -- doesn't make sense
+			title.setPreferredSize(new Dimension(list.getSize().width,15));
 			//Vector tmp2=new Vector();
 			//for(int i=0;i<1000000;i++) {tmp2.add(new Integer(0));tmp2.remove(tmp2.size()-1);}
 			if(cellHasFocus)
@@ -161,8 +168,14 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		}
 	}
  	
+	void gotConnection() {
+		updated();
+		repaint();
+	}
+
 	void lostConnection() {
 		updated();
+		repaint();
 	}
 
 	void updated() {
@@ -173,13 +186,17 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		//scripts.setEnabled(comm._isConnected);
 		title.setEnabled(comm._isConnected);
 		backBut.setEnabled(comm._isConnected);
+		estopBut.setEnabled(comm._isConnected);
 		refreshBut.setEnabled(comm._isConnected);
 		inputField.setEnabled(comm._isConnected);
-		title.removeAllItems();//clearing the title popup menu, but we'll refresh the items if we're still connected
 		if(!comm._isConnected) {
-			title.addItem("-");
-			menu.setListData(new Vector());
-			status.setText("Reconnecting...");
+			synchronized(menuScroll) {
+				title.removeAllItems();
+				title.addItem("-");
+				menu.setListData(new Vector());
+				status.setText("Reconnecting...");
+				status.setToolTipText("");
+			}
 		} else {
 			if(comm._connectCount>0) {
 				if(comm._connectCount==1) {
@@ -188,8 +205,14 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 							comm.sendInput(((ScriptEntry)scriptsModel.get(i)).cmd);
 					//restart servers for any windows we still have open
 					Iterator mons=comm.dynObjSrcs.values().iterator();
-					while(mons.hasNext())
-						comm.sendInput("!root "+(String)mons.next());
+					LinkedList started=new LinkedList(); //only start each entry once (may open multiple items)
+					while(mons.hasNext()) {
+						String c=(String)mons.next();
+						if(!started.contains(c)) {
+							comm.sendInput("!root "+c);
+							started.add(c);
+						}
+					}
 					//Set focus to the main menu
 					menu.requestFocus();
 				}
@@ -198,19 +221,38 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 						comm.sendInput(((ScriptEntry)scriptsModel.get(i)).cmd);
 				comm._connectCount=-1; //so we don't get it again
 			}
+
+			Vector menuitems;
 			synchronized(comm._menus) {
-				int len=0;
-				for(Iterator it=comm._titles.iterator(); it.hasNext();) {
+				boolean titleDirty=false;
+				int index=comm._titles.size()-1;
+				for(Iterator it=comm._titles.iterator(); index>=0 && it.hasNext();index--) {
 					String entry=(String)it.next();
-					title.insertItemAt(entry,0);
-					if(entry.length()>len)
-						len=entry.length();
+					if(!entry.equals((String)title.getItemAt(index))) {
+						titleDirty=true;
+						break;
+					}
 				}
-				title.setMaximumSize(new Dimension((int)(len*8)+25,title.getHeight()));
-				title.setSelectedIndex(0);
-				Vector menuitems=(Vector)comm._menus.lastElement();
+				if(titleDirty) {
+					title.removeAllItems();//clearing the title popup menu, but we'll refresh the items if we're still connected
+					int len=0;
+					for(Iterator it=comm._titles.iterator(); it.hasNext();) {
+						String entry=(String)it.next();
+						title.insertItemAt(entry,0);
+						if(entry.length()>len)
+							len=entry.length();
+					}
+					title.setMaximumSize(new Dimension((int)(len*8)+25,title.getHeight()));
+					title.setSelectedIndex(0);
+				}
+				menuitems=(Vector)comm._menus.lastElement();
+				menuitems=(Vector)menuitems.clone();
+				updateStatusText();
+			}
+
+			//synchronized(this) {
 				menu.setValueIsAdjusting(true);
-				menu.setListData((Vector)menuitems.clone());
+				menu.setListData(menuitems);
 				Vector sels=new Vector();
 				for(int i=0; i<menuitems.size(); i++)
 					if(((ControllerListener.MenuEntry)menuitems.get(i)).selected)
@@ -220,24 +262,42 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 					selsArr[i]=((Integer)sels.get(i)).intValue();
 				menu.setSelectedIndices(selsArr);
 				menu.setValueIsAdjusting(false);
-			}
-			status.setText(comm._status.length()>0?comm._status:"Connected.");
+			//}
 		}
 		menuScroll.getViewport().setViewPosition(p);
 		isUpdating=false;
 		comm._updatedFlag=false;
 	}
 
-	public void paint(Graphics g) {
+	/*public void paint(Graphics g) {
 		if(comm!=null && comm._updatedFlag)
 			updated();
 		super.paint(g);
+		}*/
+
+	public void estopUpdated(EStopListener l) {
+		updateStatusText();
+		if(estopBut!=null)
+			estopBut.setText(estopComm.getEStop()?"Un-Stop":"Stop!");
+	}
+	
+	public void updateStatusText() {
+		if(comm._status.length()>0) {
+			status.setText(comm._status);
+		} else {
+			if(status!=null && estopComm!=null) //otherwise still doing initial connect, leave at default text
+				status.setText(estopComm.getEStop()?"Stopped":"Running");
+		}
+		if(status!=null)
+			status.setToolTipText(comm._status);
 	}
 	
 	public void actionPerformed(ActionEvent evt) {
 		if(evt.getSource()==backBut) {
 			//System.out.println("back button clicked");
 			comm.sendReturn();
+		} else if(evt.getSource()==estopBut) {
+			estopComm.toggleEStop();
 		} else if(evt.getSource()==refreshBut) {
 			//System.out.println("refresh button clicked");
 			comm.sendRefresh();
@@ -249,7 +309,8 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		} else if(evt.getSource()==inputField) {
 			//System.out.println("input: "+inputField.getText());
 			int[] sel=menu.getSelectedIndices();
-      if (inputField.getText().length()==0) {
+			if (inputField.getText().length()==0) {
+				return;
 			} else if(sel.length==0 || inputField.getText().charAt(0)=='!')
 				comm.sendInput(inputField.getText());
 			else
@@ -261,8 +322,11 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 				for(int i=0; i<sel.length; i++)
 					comm.sendInput(inputField.getText(),comm.buildSelectionPath(sel[i]));
 			*/
-      inputFieldHistory.add(inputField.getText());
-      inputFieldLocation=inputFieldHistory.size();
+			
+			if(!inputFieldHistory.get(inputFieldHistory.size()-1).equals(inputField.getText())) {
+				inputFieldHistory.add(inputField.getText());
+			}
+			inputFieldLocation=inputFieldHistory.size();
 			inputField.setText("");
 		} else if(evt.getSource()==reconnectBut) {
 			int port=comm._port;
@@ -276,16 +340,57 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 			comm.dynObjs=dynObjs;
 			comm.dynObjSrcs=dynObjSrcs;
 			comm.dynObjPorts=dynObjPorts;
+			comm.needConnection();
 			estop.close();
 			estop.open();
-		} else if(evt.getActionCommand().equals("raw") || evt.getActionCommand().equals("rle")) {
+		} else if(evt.getActionCommand().equals("CameraSketchSpace")) {
+			if(cameraSkGUI==null || !cameraSkGUI.isDisplayable()) {
+				org.tekkotsu.sketch.SketchGUI.Space _space = org.tekkotsu.sketch.SketchGUI.Space.cam;
+				int _listingPort = 5800;
+				int _sketchPort = 5801;
+				cameraSkGUI = new org.tekkotsu.sketch.SketchGUI(comm._host, _listingPort, _sketchPort, _space);
+				cameraSkGUI.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
+				cameraSkGUI.setVisible(true);
+			} else {
+				cameraSkGUI.close();
+				cameraSkGUI=null;
+			}
+		} else if(evt.getActionCommand().equals("LocalSketchSpace")) {
+			if(localSkGUI==null || !localSkGUI.isDisplayable()) {
+				org.tekkotsu.sketch.SketchGUI.Space _space = org.tekkotsu.sketch.SketchGUI.Space.local;
+				int _listingPort = 5802;
+				int _sketchPort = 5803;
+				localSkGUI = new org.tekkotsu.sketch.SketchGUI(comm._host, _listingPort, _sketchPort, _space);
+				localSkGUI.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
+				localSkGUI.setVisible(true);
+			} else {
+				localSkGUI.close();
+				localSkGUI=null;
+			}
+		} else if(evt.getActionCommand().equals("WorldSketchSpace")) {
+			if(worldSkGUI==null || !worldSkGUI.isDisplayable()) {
+				org.tekkotsu.sketch.SketchGUI.Space _space = org.tekkotsu.sketch.SketchGUI.Space.world;
+				int _listingPort = 5804;
+				int _sketchPort = 5805;
+				worldSkGUI = new org.tekkotsu.sketch.SketchGUI(comm._host, _listingPort, _sketchPort, _space);
+				worldSkGUI.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
+				worldSkGUI.setVisible(true);
+			} else {
+				worldSkGUI.close();
+				worldSkGUI=null;
+			}
+		} else if(evt.getActionCommand().equals("raw") || evt.getActionCommand().equals("rle") || evt.getActionCommand().equals("reg")) {
 			if(evt.getActionCommand().equals("raw")) {
 				comm.sendInput("!root \"TekkotsuMon\" \"Raw Cam Server\"");
 				comm.dynObjSrcs.put("RawVisionGUI","\"TekkotsuMon\" \"Raw Cam Server\"");
-			} else {
+			} else if(evt.getActionCommand().equals("rle")) {
 				comm.sendInput("!root \"TekkotsuMon\" \"Seg Cam Server\"");
 				comm.dynObjSrcs.put("SegVisionGUI","\"TekkotsuMon\" \"Seg Cam Server\"");
+			} else {
+				comm.sendInput("!root \"TekkotsuMon\" \"Region Cam Server\"");
+				comm.dynObjSrcs.put("RegionVisionGUI","\"TekkotsuMon\" \"Region Cam Server\"");
 			}
+ 
 		} else if(evt.getActionCommand().equals("addbookmark")) {
 			int sel=comm.firstSelected();
 			if(sel==-1 || !comm._isConnected) {
@@ -345,11 +450,20 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		public Dimension getSize() { return dim; }
 	}
 
+	public class SynchPaint extends JScrollPane {
+		public Object lock=new Object();
+		
+		SynchPaint(JList l) { super(l); }
+		public void paint(Graphics g) {
+			synchronized(lock) {
+				super.paint(g);
+			}
+		}
+	}
+
 	protected void init() {
 		int spacer_size=10;
 	
-		
-		setTitle("TekkotsuMon: Controller Menus");
 		getContentPane().setLayout(new BorderLayout());
 		getContentPane().add(Box.createHorizontalStrut(spacer_size),BorderLayout.WEST);
 		getContentPane().add(Box.createHorizontalStrut(spacer_size),BorderLayout.EAST);
@@ -375,6 +489,7 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		
 		Box menuBox=Box.createVerticalBox();
 		menuScroll=new JScrollPane(menu);
+		//menuScroll.lock=this;
 		menuScroll.setAlignmentX(0.5f);
 		menuBox.add(menuScroll);
 		{
@@ -403,14 +518,27 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		statusbox.add(Box.createVerticalStrut(spacer_size-2));
 		{
 			JPanel tmp=new JPanel(new BorderLayout());
+			{
+				Box tmp2=Box.createHorizontalBox();
+				estopComm=new EStopListener(comm._host,EStopListener.defPort);
+				estopComm.addUpdatedListener(this);
+				estop=new EStopPanel(estopComm,true);
+				estop.setMyDim(new Dimension(carrows.getIconWidth(),carrows.getIconHeight()));
+				//estop.setMargin(new Insets(0,0,0,0));
+				tmp2.add(estop);
+				tmp2.add(Box.createHorizontalStrut(spacer_size));
+				tmp.add(tmp2,BorderLayout.WEST);
+			}
 			status=new JLabel("Connecting...");
 			tmp.add(status,BorderLayout.CENTER);
 			{
 				Box tmp2=Box.createHorizontalBox();
-				estop=new EStopButton(new EStopListener(comm._host,EStopListener.defPort));
-				estop.setMyDim(new Dimension(65,27));
-				estop.setMargin(new Insets(0,0,0,0));
-				tmp2.add(estop);
+				estopBut=new JButton("Stop!");
+				estopBut.setText(estopComm.getEStop()?"Un-Stop":"Stop!");
+				estopBut.addActionListener(this);
+				estopBut.setToolTipText("Toggle the emergency stop status");
+				estopBut.setEnabled(false);
+				tmp2.add(estopBut);
 				tmp2.add(Box.createHorizontalStrut(spacer_size));
 				reconnectBut=new JButton(carrows);
 				reconnectBut.setPreferredSize(new Dimension(carrows.getIconWidth(),carrows.getIconHeight()));
@@ -425,14 +553,14 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		
 		Box p=Box.createVerticalBox();
 		p.add(new JLabel("Send Input:"));
-    inputFieldHistory=new Vector();
+		inputFieldHistory=new Vector();
 		for(int i=0; i<MAX_STORE_INPUT_HIST; i++)
 			inputFieldHistory.add(prefs.get("inputFieldHistory"+i,""));
-    inputFieldLocation=inputFieldHistory.size();
+		inputFieldLocation=inputFieldHistory.size();
 		p.add(inputField=new JTextField());
 		inputField.setEnabled(false);
 		inputField.addActionListener(this);
-    inputField.addKeyListener(this);
+		inputField.addKeyListener(this);
 		inputField.setPreferredSize(new Dimension(30,inputField.getPreferredSize().height));
 		inputField.setToolTipText("Text from here is passed to selected controls, or the current control if no selection");
 
@@ -463,6 +591,33 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 			p.add(tmp);
 		}
 		
+		p.add(Box.createVerticalStrut(spacer_size));
+		{
+			Box tmp=Box.createHorizontalBox();
+			tmp.add(new JLabel("Sketch: "));
+			JButton but;
+		 	but=new JButton("C");
+			but.setToolTipText("Display Camera Sketch Space");
+			but.addActionListener(this);
+			but.setActionCommand("CameraSketchSpace");
+			but.setPreferredSize(new Dimension(35,22));
+			tmp.add(but);
+		 	but=new JButton("L");
+			but.setToolTipText("Display Local Sketch Space");
+			but.addActionListener(this);
+			but.setActionCommand("LocalSketchSpace");
+			but.setPreferredSize(new Dimension(35,22));
+			tmp.add(but);
+		 	but=new JButton("W");
+			but.setToolTipText("Display World Sketch Space");
+			but.addActionListener(this);
+			but.setActionCommand("WorldSketchSpace");
+			but.setPreferredSize(new Dimension(35,22));
+			tmp.add(but);
+			tmp.setAlignmentX(0.0f);
+			p.add(tmp);
+		}
+		
 		p.add(Box.createVerticalStrut(spacer_size));
 		{
 			JSeparator sep=new JSeparator(SwingConstants.HORIZONTAL);
@@ -518,10 +673,17 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
 		addWindowListener(new CloseVisionAdapter(this));
 
 		setLocation(prefs.getInt("ControllerGUI.location.x",50),prefs.getInt("ControllerGUI.location.y",50));
-		int numScripts=prefs.getInt("ControllerGUI.numScripts",0);
-		for(int i=0; i<numScripts; i++)
-			scriptsModel.addElement(new ScriptEntry(prefs.get("ControllerGUI.script"+i+".title","unknown"),prefs.get("ControllerGUI.script"+i+".cmd","")));
-
+		int numScripts=prefs.getInt("ControllerGUI.numScripts",-1);
+		if(numScripts>-1) {
+			for(int i=0; i<numScripts; i++)
+				scriptsModel.addElement(new ScriptEntry(prefs.get("ControllerGUI.script"+i+".title","unknown"),prefs.get("ControllerGUI.script"+i+".cmd","")));
+		} else {
+			//uninitialized, supply some default scripts
+			scriptsModel.addElement(new ScriptEntry("Quality Video","!set vision.rawcam_transport=tcp\n!set vision.rawcam_compression=none\n!set vision.rawcam_y_skip=1\n!set vision.rawcam_uv_skip=1\n!set vision.rawcam_interval=1500"));
+			scriptsModel.addElement(new ScriptEntry("Smooth Video","!set vision.rawcam_y_skip=2\n!set vision.rawcam_uv_skip=3\n!set vision.rawcam_interval=0\n!set vision.rawcam_compression=jpeg\n!set vision.rawcam_transport=udp"));
+			scriptsModel.addElement(new ScriptEntry("Head Remote Control","!root \"TekkotsuMon\" \"Head Remote Control\""));
+			scriptsModel.addElement(new ScriptEntry("Walk Remote Control","!root \"TekkotsuMon\" \"Walk Remote Control\""));
+		}
 		pack();
 		title.setMaximumSize(new Dimension(15+25,title.getHeight()));
 		setVisible(true);
@@ -599,14 +761,20 @@ public class ControllerGUI extends JFrame implements ActionListener, KeyListener
       int key=e.getKeyCode();
       if (key==KeyEvent.VK_UP || key==KeyEvent.VK_KP_UP) {
         inputFieldLocation--;
-        if (inputFieldLocation < 0)
-          inputFieldLocation+=inputFieldHistory.size();
+        if (inputFieldLocation < 0) {
+          java.awt.Toolkit.getDefaultToolkit().beep();
+          inputFieldLocation=0;
+        }
         inputField.setText((String)inputFieldHistory.get(inputFieldLocation));
       } else if (key==KeyEvent.VK_DOWN || key==KeyEvent.VK_KP_DOWN) {
         inputFieldLocation++;
-        if (inputFieldLocation >= inputFieldHistory.size())
-          inputFieldLocation-=inputFieldHistory.size();
-        inputField.setText((String)inputFieldHistory.get(inputFieldLocation));
+        if (inputFieldLocation == inputFieldHistory.size()) {
+          inputField.setText("");
+        } else if (inputFieldLocation > inputFieldHistory.size()) {
+          java.awt.Toolkit.getDefaultToolkit().beep();
+          inputFieldLocation=inputFieldHistory.size();
+        } else
+          inputField.setText((String)inputFieldHistory.get(inputFieldLocation));
       }
     }
   }
