@@ -53,7 +53,7 @@ public class EStopButton extends JButton implements EStopListener.UpdatedListene
 		setEnabled(mode!=DISABLED_MODE);
 		comm.addUpdatedListener(this);
 		
-		setToolTipText("Toggle Emergency Stop; alt-click to open new window");
+		setToolTipText("Toggle Emergency Stop; middle-click (or alt-click) to open into new window");
 		enableEvents(AWTEvent.MOUSE_EVENT_MASK);
 
 		updateStatus();
