@@ -14,9 +14,9 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 	JSlider pslide;
 	JSlider rslide;
 	JButton stopBut;
-	JRadioButton horizRollBut;
-	JRadioButton horizPanBut;
-	boolean horizButFake=false;
+	JRadioButton vertNodBut;
+	JRadioButton vertTiltBut;
+	boolean vertButFake=false;
 	JCheckBox resetOnRelease;
 	JLabel status;
 	JButton reconnectBut;
@@ -54,7 +54,7 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 		int offx=getGraphicsConfiguration().getBounds().x;
 		int offy=getGraphicsConfiguration().getBounds().y;
 		setLocation(prefs.getInt("HeadPointGUI.location.x",50),prefs.getInt("HeadPointGUI.location.y",50));
-		show();
+		setVisible(true);
 	}
 
 	public void close() {
@@ -77,18 +77,18 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 
 	public void pointPicked(Point2D.Float p, MouseEvent e, PointPick pp) {
 		boolean isBut2=(e.getModifiersEx()&MouseEvent.BUTTON3_DOWN_MASK)==MouseEvent.BUTTON3_DOWN_MASK;
-		if(!horizButFake && isBut2) {
-			if(horizPanBut.isSelected())
-				horizRollBut.setSelected(true);
+		if(!vertButFake && isBut2) {
+			if(vertTiltBut.isSelected())
+				vertNodBut.setSelected(true);
 			else
-				horizPanBut.setSelected(true);
-			horizButFake=isBut2;
+				vertTiltBut.setSelected(true);
+			vertButFake=isBut2;
 		}
-		if(horizRollBut.isSelected())
-			rslide.setValue((int)(slidermax*p.x));
-		if(horizPanBut.isSelected())
-			pslide.setValue((int)(slidermax*p.x));
-		tslide.setValue((int)(slidermax*p.y));
+		if(vertNodBut.isSelected())
+			rslide.setValue((int)(slidermax*p.y));
+		if(vertTiltBut.isSelected())
+			tslide.setValue((int)(slidermax*p.y));
+		pslide.setValue((int)(slidermax*p.x));
 	}
 
 	public void headPointUpdated(HeadPointListener comm) {
@@ -110,24 +110,24 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 	public void mouseExited(MouseEvent e) {}
 	public void mousePressed(MouseEvent e) {
 		boolean isBut2=(e.getModifiersEx()&MouseEvent.BUTTON3_DOWN_MASK)==MouseEvent.BUTTON3_DOWN_MASK;
-		if(!horizButFake && isBut2) {
-			if(horizPanBut.isSelected())
-				horizRollBut.setSelected(true);
+		if(!vertButFake && isBut2) {
+			if(vertTiltBut.isSelected())
+				vertNodBut.setSelected(true);
 			else
-				horizPanBut.setSelected(true);
-			horizButFake=isBut2;
+				vertTiltBut.setSelected(true);
+			vertButFake=isBut2;
 			updatePP();
 		}
 	}
 	public void mouseReleased(MouseEvent e) {
 		boolean isBut1=(e.getModifiersEx()&MouseEvent.BUTTON1_DOWN_MASK)==MouseEvent.BUTTON1_DOWN_MASK;
 		boolean isBut2=(e.getModifiersEx()&MouseEvent.BUTTON3_DOWN_MASK)==MouseEvent.BUTTON3_DOWN_MASK;
-		if(horizButFake && !isBut2) {
-			if(horizPanBut.isSelected())
-				horizRollBut.setSelected(true);
+		if(vertButFake && !isBut2) {
+			if(vertTiltBut.isSelected())
+				vertNodBut.setSelected(true);
 			else
-				horizPanBut.setSelected(true);
-			horizButFake=isBut2;
+				vertTiltBut.setSelected(true);
+			vertButFake=isBut2;
 			updatePP();
 		}
 		if(!isBut1 && !isBut2) {
@@ -139,18 +139,18 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 	public void stateChanged(ChangeEvent e) {
 		if(e.getSource()==tslide) {
 			comm.sendCommand("t",tslide.getValue()/(float)slidermax);
-			pp.doSetPoint(pp.getXValue(),tslide.getValue()/(float)slidermax);
+			if(vertTiltBut.isSelected())
+				pp.doSetPoint(pp.getXValue(),tslide.getValue()/(float)slidermax);
 		} else if(e.getSource()==pslide) {
 			comm.sendCommand("p",-pslide.getValue()/(float)slidermax);
-			if(horizPanBut.isSelected())
-				pp.doSetPoint(pslide.getValue()/(float)slidermax,pp.getYValue());
+			pp.doSetPoint(pslide.getValue()/(float)slidermax,pp.getYValue());
 		} else if(e.getSource()==rslide) {
 			// Rotation is both fast and sensitive, so we'll exponentiate it to
 			// drag out the low end without sacrificing the high end
 			comm.sendCommand("r",rslide.getValue()/(float)slidermax);
 			float tmp=pp.getYValue();
-			if(horizRollBut.isSelected())
-				pp.doSetPoint(rslide.getValue()/(float)slidermax,pp.getYValue());
+			if(vertNodBut.isSelected())
+				pp.doSetPoint(pp.getXValue(),rslide.getValue()/(float)slidermax);
 		}
 	}
 
@@ -159,9 +159,9 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 			tslide.setValue(0);
 			pslide.setValue(0);
 			rslide.setValue(0);
-		} else if(e.getSource()==horizRollBut) {
+		} else if(e.getSource()==vertNodBut) {
 			updatePP();
-		} else if(e.getSource()==horizPanBut) {
+		} else if(e.getSource()==vertTiltBut) {
 			updatePP();
 		} else if(e.getSource()==reconnectBut) {
 			int port=comm._port;
@@ -174,12 +174,12 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 	}
 	
 	public void updatePP() {
-		float x=0;
-		if(horizPanBut.isSelected())
-			x=pslide.getValue()/(float)slidermax;
-		else if(horizRollBut.isSelected())
-			x=rslide.getValue()/(float)slidermax;
-		pp.doSetPoint(x,tslide.getValue()/(float)slidermax);
+		float y=0;
+		if(vertTiltBut.isSelected())
+			y=tslide.getValue()/(float)slidermax;
+		else if(vertNodBut.isSelected())
+			y=rslide.getValue()/(float)slidermax;
+		pp.doSetPoint(pslide.getValue()/(float)slidermax,y);
 	}
 
 	public void frameInit() {
@@ -252,19 +252,19 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 				tmp2.add(tmp3);
 			}
 			tmp2.add(Box.createVerticalStrut(strutsize));
-			tmp2.add(new JLabel("Roll:"));
+			tmp2.add(new JLabel("Nod:"));
 			{
 				Box tmp3=Box.createHorizontalBox();
 				rslide=new JSlider(-slidermax,slidermax,0);
 				rslide.addChangeListener(this);
 				JLabel lab;
-				lab=new JLabel("Counter");
+				lab=new JLabel("Down");
 				lab.setFont(lab.getFont().deriveFont(lab.getFont().getSize2D()-2));
 				lab.setHorizontalAlignment(SwingConstants.RIGHT);
 				lab.setPreferredSize(new Dimension(labwidth,lab.getFont().getSize()));
 				tmp3.add(lab);
 				tmp3.add(rslide);
-				lab=new JLabel("Clock");
+				lab=new JLabel("Up");
 				lab.setFont(lab.getFont().deriveFont(lab.getFont().getSize2D()-2));
 				lab.setHorizontalAlignment(SwingConstants.LEFT);
 				lab.setPreferredSize(new Dimension(labwidth,lab.getFont().getSize()));
@@ -287,15 +287,15 @@ public class HeadPointGUI extends JFrame implements PointPick.PointPickedListene
 			}
 			tmp2.add(Box.createVerticalStrut(strutsize));
 			ButtonGroup bg = new ButtonGroup();
-			horizPanBut=new JRadioButton("Horizontal is Pan");
-			horizPanBut.addActionListener(this);
-			horizPanBut.setSelected(true);
-			bg.add(horizPanBut);
-			tmp2.add(horizPanBut);
-			horizRollBut=new JRadioButton("Horizontal is Roll");
-			horizRollBut.addActionListener(this);
-			bg.add(horizRollBut);
-			tmp2.add(horizRollBut);
+			vertTiltBut=new JRadioButton("Vertical is Tilt");
+			vertTiltBut.addActionListener(this);
+			bg.add(vertTiltBut);
+			tmp2.add(vertTiltBut);
+			vertNodBut=new JRadioButton("Vertical is Nod");
+			vertNodBut.addActionListener(this);
+			bg.add(vertNodBut);
+			tmp2.add(vertNodBut);
+			vertNodBut.setSelected(true);
 			tmp2.add(Box.createVerticalStrut(strutsize));
 			tmp2.add(resetOnRelease=new JCheckBox("Center on release")); 
 			tmp2.add(Box.createVerticalGlue());
