@@ -45,6 +45,9 @@ public interface VisionListener {
 	void fireVisionUpdate();
 	public boolean isConnected();
 
+	public void startSensors();
+	public void stopSensors();
+
 	public Date getTimeStamp();
 	public long getFrameNum();
 	public IndexColorModel getColorModel();
@@ -53,7 +56,8 @@ public interface VisionListener {
 	public byte[] getJPEG();
 	public int getJPEGLen();
 	
-    public byte[] getData();
+	public byte[] getData();
+	public String getSensors();
 	public void setConvertRGB(boolean b);
 	public boolean getConvertRGB();
 
