@@ -26,7 +26,7 @@ public class VisionPanel extends JPanel implements VisionUpdatedListener,
                                                    MouseListener,
 						   MouseMotionListener
 {
-    boolean usingSketchGUI;
+    boolean usingSketchGUI=false;
 
     protected boolean windowHasFocus;
     protected boolean crosshairsEnabled=false;
@@ -38,8 +38,8 @@ public class VisionPanel extends JPanel implements VisionUpdatedListener,
     float tgtAspect=-1;
 
     SketchGUI gui;
-    //    boolean isCam;
-    int space;
+    protected SketchGUI.Space space;
+    //    public int space;
 
     int drawX, drawY, drawHeight, drawWidth;
 
@@ -77,15 +77,13 @@ public class VisionPanel extends JPanel implements VisionUpdatedListener,
 	public void setConvertRGB(boolean b) { _listener.setConvertRGB(b); }
 	public boolean getConvertRGB() { return _listener.getConvertRGB(); }
 
-    //	public VisionPanel(VisionListener listener, SketchGUI _gui, boolean _isCam) {
-	public VisionPanel(VisionListener listener, SketchGUI _gui, int _space) {
-		super();
-		init(listener);
-		gui = _gui;
-		//isCam = _isCam;
-		space = _space;
+    //	public VisionPanel(VisionListener listener, SketchGUI _gui, int _space) {
+        public VisionPanel(VisionListener listener, SketchGUI _gui, SketchGUI.Space _space) {
+	    super();
 		usingSketchGUI = true;
-		_listener.removeListener(this);
+	    init(listener);
+	    gui = _gui;
+	    space = _space;
 	}
 	
 	public VisionPanel(String host, String[] args) {
@@ -136,7 +134,6 @@ public class VisionPanel extends JPanel implements VisionUpdatedListener,
 	}
 	
 	public void init(VisionListener listener) {
-	    usingSketchGUI = false;
 	    setBackground(Color.BLACK);
 	    setForeground(Color.WHITE);
 	    setOpaque(!lockAspect);
@@ -145,14 +142,16 @@ public class VisionPanel extends JPanel implements VisionUpdatedListener,
 	    windowHasFocus = true;
 	    crosshairsEnabled = false;
 	    _listener=listener;
-	    _listener.addListener(this);
+		_listener.addListener(this);
 	}
 
    
 	public VisionListener getListener() { return _listener; }
-	public void close() { _listener.kill();	}
+	public void close() { _listener.close();	}
+	public void kill() { _listener.kill(); }
 	public void open() { _listener.needConnection(); }
 	public void visionUpdated(VisionListener l) { repaint(); }
+	public void sensorsUpdated(VisionListener l) {}
 	
 	public void setLockAspectRatio(boolean b) {
 		if(b!=lockAspect) {
@@ -180,63 +179,63 @@ public class VisionPanel extends JPanel implements VisionUpdatedListener,
 		setAspectRatio(asp);
 	}
 	
-  public void paint(Graphics graphics) {
-      if (!usingSketchGUI) { _image=_listener.getImage(); }
-      super.paint(graphics);
-      Dimension sz=getSize();
-      drawX = 0;
-      drawY = 0;
-      drawWidth = sz.width;
-      drawHeight = sz.height;
-      // Scale image to fit the window size while maintaining aspect ratio.
-      // Center the image in the unused space if window width or height too large.
-      // Note: this only makes sense for cam space.
-      //      if (_image != null && getLockAspectRatio() && (!usingSketchGUI || isCam)) {
-      if (_image != null && getLockAspectRatio() && (!usingSketchGUI || space==1)) {
-	  float curasp=sz.width/(float)sz.height;
-	  float tgtasp=getAspectRatio();
-	  if(tgtasp<0)
-	      tgtasp=_image.getWidth()/(float)_image.getHeight();
-	  if(curasp>tgtasp) {
-	      drawWidth = (int)(sz.height*tgtasp);
-	      drawX = (sz.width-drawWidth)/2;
-	  } else if(curasp<tgtasp) {
-	      drawHeight = (int)(sz.width/tgtasp);
-	      drawY = (sz.height-drawHeight)/2;
-	  } else {
-	  }
-      }
-      drawImage(graphics, _image, drawX, drawY, drawWidth, drawHeight);
-      
-      // If requested, draw crosshairs for RawCam, SegCam, or cam sketchspace.
-      // Crosshairs for world sketchspace are handled in SketchPanel.java.
-      //      if (crosshairsEnabled && (!usingSketchGUI || isCam))
-      if (crosshairsEnabled && (!usingSketchGUI || space==1))
-	  {
-	      graphics.setXORMode(Color.GRAY);
-	      graphics.setColor(Color.WHITE);
-	      ((Graphics2D)graphics).setStroke(new BasicStroke(1.0f));
-	      graphics.drawLine(drawX+drawWidth/2,drawY, drawX+drawWidth/2, drawY+drawHeight);
-	      graphics.drawLine(drawX, drawY+drawHeight/2, drawX+drawWidth, drawY+drawHeight/2);
-	      graphics.setPaintMode();
-	  }
-  }
-
-    protected void drawImage(Graphics g, BufferedImage img, int x, int y, int w, int h) {
-	if(img!=null)
-	    g.drawImage(img,x,y,w,h,null);
-	else {
-	    g.setColor(getBackground());
-	    g.fillRect(x,y,w,h);
-	    FontMetrics tmp=g.getFontMetrics();
-	    String msg="No image";
-	    int strw=tmp.stringWidth(msg);
-	    int strh=tmp.getHeight();
-	    g.setColor(getForeground());
-	    g.drawString(msg,(getSize().width-strw)/2,(getSize().height-strh)/2+tmp.getAscent());
+	public void paint(Graphics graphics) {
+		if (!usingSketchGUI) { _image=_listener.getImage(); }
+		super.paint(graphics);
+		Dimension sz=getSize();
+		drawX = 0;
+		drawY = 0;
+		drawWidth = sz.width;
+		drawHeight = sz.height;
+		// Scale image to fit the window size while maintaining aspect ratio.
+		// Center the image in the unused space if window width or height too large.
+		// Note: this only makes sense for cam space.
+		if (_image != null && getLockAspectRatio() && (!usingSketchGUI || space==SketchGUI.Space.cam)) {
+			float curasp=sz.width/(float)sz.height;
+			float tgtasp=getAspectRatio();
+			if(tgtasp<0)
+				tgtasp=_image.getWidth()/(float)_image.getHeight();
+			if(curasp>tgtasp) {
+				drawWidth = (int)(sz.height*tgtasp);
+				drawX = (sz.width-drawWidth)/2;
+			} else if(curasp<tgtasp) {
+				drawHeight = (int)(sz.width/tgtasp);
+				drawY = (sz.height-drawHeight)/2;
+			} else {
+			}
+		}
+		drawImage(graphics, _image, drawX, drawY, drawWidth, drawHeight);
+		
+		// If requested, draw crosshairs for RawCam, SegCam.
+		// Crosshairs for SketchGUI are handled in SketchPanel.java.
+		if (crosshairsEnabled && !usingSketchGUI)
+		{
+			graphics.setXORMode(Color.GRAY);
+			graphics.setColor(Color.WHITE);
+			((Graphics2D)graphics).setStroke(new BasicStroke(1.0f));
+			graphics.drawLine(drawX+drawWidth/2,drawY, drawX+drawWidth/2, drawY+drawHeight);
+			graphics.drawLine(drawX, drawY+drawHeight/2, drawX+drawWidth, drawY+drawHeight/2);
+			graphics.setPaintMode();
+		}
 	}
-    }
-
+	
+	protected void drawImage(Graphics g, BufferedImage img, int x, int y, int w, int h) {
+		if(img!=null) {
+			synchronized(img) {
+				g.drawImage(img,x,y,w,h,null);
+			}
+		} else {
+			g.setColor(getBackground());
+			g.fillRect(x,y,w,h);
+			FontMetrics tmp=g.getFontMetrics();
+			String msg="No image";
+			int strw=tmp.stringWidth(msg);
+			int strh=tmp.getHeight();
+			g.setColor(getForeground());
+			g.drawString(msg,(getSize().width-strw)/2,(getSize().height-strh)/2+tmp.getAscent());
+		}
+	}
+	
     public void mouseDragged(MouseEvent e){}
 
     public void mouseMoved(MouseEvent e)
