@@ -6,10 +6,10 @@ import javax.swing.Icon;
 import javax.swing.ImageIcon;
 
 public class AgentShapeInfo extends ShapeInfo {
-	static Icon icon = new ImageIcon("org/tekkotsu/mon/icons/agent.png");
+	static Icon icon = new ImageIcon(icon_path+"agent.png");
 	float orientation; // orientation of agent
 
-	public AgentShapeInfo(int _id, int _parentId, String _name, Color _color,
+    public AgentShapeInfo(int _id, int _parentId, String _name, Color _color,
 			float _centroidx, float _centroidy, float _centroidz,
 			float _orientation) {
 		super(_id, _parentId, _name, _color, _centroidx,_centroidy, _centroidz);
@@ -31,9 +31,27 @@ public class AgentShapeInfo extends ShapeInfo {
 	
 	public Icon getIcon() { return icon; }
 
-	public void renderTo(Graphics2D graphics) {
-		graphics.drawOval((int)(getCentroidX()+1),
-			(int)(getCentroidY()+1), 5, 5);
+    public void renderTo(Graphics2D graphics, float scaling) {
+	    //		graphics.drawOval((int)(getCentroidX()+1),
+	    //	(int)(getCentroidY()+1), 5, 5);
+		int [] coordsX, coordsY;
+		coordsX = new int[3];
+		coordsY = new int[3];
+		
+		//represent the dog as a triangle which has base length of 10cm, height of 20cm
+		//top
+		coordsX[0] = (int) (centroidx + 100 * Math.cos(orientation));
+		coordsY[0] = (int) (centroidy + 100 * Math.sin(orientation));
+
+		//left
+		coordsX[1] = (int) (centroidx + 120 * Math.cos(-2.618+orientation));
+		coordsY[1] = (int) (centroidy + 120 * Math.sin(-2.618+orientation));
+
+		//right
+		coordsX[2] = (int) (centroidx + 120 * Math.cos(2.618+orientation));
+		coordsY[2] = (int) (centroidy + 120 * Math.sin(2.618+orientation));
+		graphics.drawPolygon(coordsX,coordsY, 3);
+
 	}
 
 	public float getOrientation() { return orientation; }
